==> e2e-cypress-dsl-10.1.3.zip: dsl/pages/permissions-page.js <==
import TagEditor from '../components/tag-editor';

export default class PermissionsPage {

    constructor() {
        cy.get('#permissions .table').should('be.visible');
    }

    add(role, permission) {
        const permissionEditor = new TagEditor(`tr:contains('${permission}')`);
        permissionEditor.addTag(`${role}`);
        return this;
    }

    save() {
        cy.get(".save").click();
        cy.wait(1000);
        return this;
    }


    reset() {
        cy.get(".reset").click();
        return this;
    }

    remove(role, permission) {
        cy.get(`tr:contains('${permission}') li:contains('${role}') .tag-close`).click();
        return this;
    }

    refresh() {
        cy.reload();
        return this;
    }

    expectPermissions(...definitions) {
        for (const definition of definitions) {

            Object.keys(definition.permissions).forEach((permission) => {
                let count = 0;
                if (definition.permissions[permission]) {
                    count = 1;
                }
                cy.get(`tr:contains('${permission}') td:contains('${definition.role}')`).should('have.length',count);
            });
        }
        return this;
    }

    expectResetEnabled(enabled = true) {
        if (enabled) {
            cy.get('.button.reset').should('have.length',1);
            // expect(element('.button.reset').count()).toBe(1);
            cy.get('.button.reset[disabled="disabled"]').should('have.length',0);
            // expect(element('.button.reset[disabled="disabled"]').count()).toBe(0);
        } else {
            cy.get('.button.reset[disabled="disabled"]').should('have.length',1);
            // expect(element('.button.reset[disabled="disabled"]').count()).toBe(1);
        }
        return this;
    }

    expectSaveEnabled(enabled = true) {
        if (enabled) {
            cy.get('.button.save').should('have.length',1);
            // expect(element('.button.reset').count()).toBe(1);
            cy.get('.button.save[disabled="disabled"]').should('have.length',0);
            // expect(element('.button.save').count()).toBe(1);
            // expect(element('.button.save[disabled="disabled"]').count()).toBe(0);
        } else {
            cy.get('.button.save[disabled="disabled"]').should('have.length',1);
            // expect(element('.button.save[disabled="disabled"]').count()).toBe(1);
        }
        return this;
    }

    addRoleToPermission(role, permission) {
        let lastSaved = '';
        cy
            .waitWithoutFailing('span.last-saved:visible')
            .then(() => {
                const el = Cypress.$('span.last-saved:visible');
                if (el && el.length) {
                    lastSaved = el.text();
                    // since we have no idea when previous save happened, and we do not want to be flaky,
                    // we want next save to have different "last save" time, so that we understand, that
                    // save happened. But save has only seconds, not milliseconds, thus let's wait for
                    // 2 seconds to make sure, that new time will be different.
                    // we need to wait couple of seconds to make last save time different to previous one
                    return new Promise((res) => {
                        setTimeout(res, 2000);
                    });
                }
            });
        const input = cy.get(`tr:contains('${permission}') .tag-input`).first();
        input.type(role);
        input.type('{enter}');
        cy.get(".save:visible").click()
            .waitWithoutFailing(() => {
                const el = Cypress.$('span.last-saved:visible');
                if (!el.length || el.text() === lastSaved) {
                    throw new Error('not saved yet');
                }
            })
    }

    addAdminPermission(role) {
        this.addRoleToPermission(role, 'Admin');
        return this;
    }

    addEditSecurityPermission(role) {
        this.addRoleToPermission(role, 'Edit security');
        return this;
    }

    addCreateTemplatePermission(role) {
        this.addRoleToPermission(role, 'Create template');
        return this;
    }

    addEditGlobalVariablesPermission(role) {
        this.addRoleToPermission(role, 'Edit global variables');
        return this;
    }

    addCreateTopLevelFoldersPermission(role) {
        this.addRoleToPermission(role, 'Create top level folders');
        return this;
    }

    addAndSaveAsCreateDashboard(role) {
        this.addRoleToPermission(role, 'Create dashboard');
        return this;
    }

    addAndSaveAsViewReports(role) {
        this.addRoleToPermission(role, 'View reports');
        return this;
    }
}

==> e2e-cypress-dsl-22.2.8.zip: dsl/pages/permissions-page.js <==
import TagEditor from '../components/tag-editor';

export default class PermissionsPage {
    constructor() {
        cy.get('#permissions .table').should('be.visible');
    }

    add(role, permission) {
        const permissionEditor = new TagEditor(`tr:contains('${permission}')`);
        permissionEditor.addTag(`${role}`);
        return this;
    }

    save() {
        cy.get("action-toolbar button:contains('Save')").click();
        cy.wait(1000);
        return this;
    }

    reset() {
        cy.get("action-toolbar button:contains('Reset')").click();
        return this;
    }

    remove(role, permission) {
        cy.get(`tr:contains('${permission}') li:contains('${role}') .tag-close`).click();
        return this;
    }

    refresh() {
        cy.reload();
        return this;
    }

    expectPermissions(...definitions) {
        for (const definition of definitions) {
            Object.keys(definition.permissions).forEach((permission) => {
                let count = 0;
                if (definition.permissions[permission]) {
                    count = 1;
                }
                cy.get(`tr:contains('${permission}') td:contains('${definition.role}')`).should('have.length', count);
            });
        }
        return this;
    }

    expectResetEnabled(enabled = true) {
        if (enabled) {
            cy.get("action-toolbar button:contains('Reset')").should('have.length', 1);
            // expect(element('.button.reset').count()).toBe(1);
            cy.get("action-toolbar button[disabled='disabled']:contains('Reset')").should('have.length', 0);
            // expect(element('.button.reset[disabled="disabled"]').count()).toBe(0);
        } else {
            cy.get("action-toolbar button[disabled='disabled']:contains('Reset')").should('have.length', 1);
            // expect(element('.button.reset[disabled="disabled"]').count()).toBe(1);
        }
        return this;
    }

    expectSaveEnabled(enabled = true) {
        if (enabled) {
            cy.get("action-toolbar button:contains('Save')").should('have.length', 1);
            // expect(element('.button.reset').count()).toBe(1);
            cy.get("action-toolbar button[disabled='disabled']:contains('Save')").should('have.length', 0);
            // expect(element('.button.save').count()).toBe(1);
            // expect(element('.button.save[disabled="disabled"]').count()).toBe(0);
        } else {
            cy.get("action-toolbar button[disabled='disabled']:contains('Save')").should('have.length', 1);
            // expect(element('.button.save[disabled="disabled"]').count()).toBe(1);
        }
        return this;
    }

    addRoleToPermission(role, permission) {
        let lastSaved = '';
        cy.waitWithoutFailing('span.saved-text:visible').then(() => {
            const el = Cypress.$('span.saved-text:visible');
            if (el && el.length) {
                lastSaved = el.text();
                // since we have no idea when previous save happened, and we do not want to be flaky,
                // we want next save to have different "last save" time, so that we understand, that
                // save happened. But save has only seconds, not milliseconds, thus let's wait for
                // 2 seconds to make sure, that new time will be different.
                // we need to wait couple of seconds to make last save time different to previous one
                return new Promise((res) => {
                    setTimeout(res, 2000);
                });
            }
        });
        const input = cy.get(`tr:contains('${permission}') .tag-input`).first();
        input.type(role);
        input.type('{enter}');
        cy.get("action-toolbar button:contains('Save')")
            .click()
            .waitWithoutFailing(() => {
                const el = Cypress.$('span.saved-text:visible');
                if (!el.length || el.text() === lastSaved) {
                    throw new Error('not saved yet');
                }
            });
    }

    addAdminPermission(role) {
        this.addRoleToPermission(role, 'Admin');
        return this;
    }

    addEditSecurityPermission(role) {
        this.addRoleToPermission(role, 'Edit security');
        return this;
    }

    addCreateTemplatePermission(role) {
        this.addRoleToPermission(role, 'Create template');
        return this;
    }

    addEditGlobalVariablesPermission(role) {
        this.addRoleToPermission(role, 'Edit global variables');
        return this;
    }

    addCreateTopLevelFoldersPermission(role) {
        this.addRoleToPermission(role, 'Create top level folders');
        return this;
    }

    addAndSaveAsCreateDashboard(role) {
        this.addRoleToPermission(role, 'Create dashboard');
        return this;
    }

    addAndSaveAsViewReports(role) {
        this.addRoleToPermission(role, 'View reports');
        return this;
    }
}
